import IOption from './IOption';
import verifyAndStoreOptionValue from './verifyAndStoreOptionValue';

export default function parseArgument(
  { spec, fail, outputArgs, requiredOptions, shortToLongLookup }: {
    spec: { [key: string]: IOption },
    fail: (errorMessage: string) => void,
    outputArgs: { [key: string]: string },
    requiredOptions: { [key: string]: boolean },
    shortToLongLookup: { [key: string]: string },
  },
  option: string,
  arg: string,
): string {
  if (option === '') {
    /**  We expect a name, must be one of:
     * - `--name value`
     * - `--name=value`
     * - `-n value`
     * - `-n=value`
     */
    if (arg === '-' || arg === '--') {
      fail(`Empty option "-" or "--" is not supported.`);
    } else if (arg.startsWith('--')) { //long argument
      option = arg.substr(2);
      let value;
      //If there's a =, immediately read the value
      if (option.indexOf('=') !== -1) {
        [option, value] = option.split('=', 2);
      }
      //Check that argument name is valid
      if (spec[option] === undefined) {
        fail(`Unknown option "--${option}".`);
      }
      //If value was provided, check that value is correct and remove name for next loop iteration
      if (value !== undefined) {
        verifyAndStoreOptionValue({option, value, verify: spec[option].verify, message: spec[option].message, fail, outputArgs, requiredOptions});
        option = '';
      }
    } else if (arg.startsWith('-')) { //short argument
      option = arg.substr(1);
      let value;
      //If there's a =, immediately read the value
      if (option.indexOf('=') !== -1) {
        [option, value] = option.split('=', 2);
      }
      //Check that argument name is valid
      if (shortToLongLookup[option] === undefined) {
        fail(`Unknown short option "-${option}".`);
      }
      option = shortToLongLookup[option];
      //If value was provided, check that value is correct and remove name for next loop iteration
      if (value !== undefined) {
        verifyAndStoreOptionValue({option, value, verify: spec[option].verify, message: spec[option].message, fail, outputArgs, requiredOptions});
        option = '';
      }
    } else {
      fail(`Arguments must be preceded by an option but there was no option in front of "${arg}".`);
    }
  } else { //We expect a value, can be anything
    verifyAndStoreOptionValue({option, value: arg, verify: spec[option].verify, message: spec[option].message, fail, outputArgs, requiredOptions});
    option = '';
  }

  return option;
}
